/*
    PGconnection.java - A Java class to manage Postgres95 connection
    Copyright (C) 1996  John F. Kelly

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

package postgres;

import java.lang.*;
import java.util.*;

/** 
 * PGrecord - a class to organize fields into records
 */

public
class PGrecord  extends Observable  {

	private boolean _dirty;
	private Hashtable _hIndexByFieldName;
	public PGfield field[];
	public String data[];

	/**
	 * This constructor is called by <a href="postgres.PGresult.html#getPGvector()">
	 * PGresult </a>
	 */
	public PGrecord(PGfield field[], Hashtable hIndexByFieldName, String data[]) {
		_dirty = false;
		this._hIndexByFieldName = hIndexByFieldName;
		this.field = field;
		this.data = data;
	}
	/** 
	 * setValue - replaces the value associated with a field name in the
	 * PGrecord.
	 * @param name - field name identifying the value to replace, value - Object to replace with
	 * @return The object that was replaced
	 * @exception PGexception if the field name isn't known
	 */
	public Object setValue(String name, Object value) {
		Object o = _hIndexByFieldName.get(name);
		if(o==null) { throw new PGexception("Unknown name "+name); }
		int i= ((Integer)(o)).intValue();
		o = data[i];
		data[i] = value.toString();
		_dirty = true;
		return o;
	}
		
	/** 
	 * getValue - gets a value Object by field name
	 * @param name - string name of the field whose value will be returned
	 * @return value as Object
	 * @exception PGexception if the field name isn't known
	 */
	
	public Object getValue(String name) {
		Object o = _hIndexByFieldName.get(name);
		if(o==null) { throw new PGexception("Unknown name "+name); }
		int i= ((Integer)(o)).intValue();
		if(data[i]==null) return null;
		switch(field[i].type) {
		case 4:
			try {
				Integer itry = new Integer(data[i]);
				return (new Integer(data[i])) ;
			} catch( NumberFormatException ne) {
				
				try {
					Date datetry =  new Date(data[i]);
					return datetry;
				} catch (Exception e) {
					return data[i];
				}
			}
		default:
			return data[i];
		}
	}
		
	/** 
	 * Returns true if record has been updated via setObject
	 */
	public boolean needsUpdate() {
		return _dirty;
	}
	/** 
	 * Returns a field's value, unquoted for integers, quoted otherwise
	 */
	public String formatValue(int i) {
		switch(field[i].type) {
		case 4:
			try {
				Integer inttry = new Integer(data[i]);
				return data[i];
			} catch ( NumberFormatException e) {
				return "'"+data[i]+"'";
			}
		default:
			return "'"+data[i]+"'";
		}
	}

	/**
	 * Returns an SQL string that will update a database
	 * table with the current value of this record
	 * for all records having the same values in key fields
	 * as this record.
	 * @param sTable - string naming the table to update
	 * 	sKeys - an array of field names identifying a "primary key"
	 * @return SQL update string
	 */
	public String updateSQL(String sTable, String keys[]) {
		int i, j;
		Object o;
		String ret = new String("update "+sTable+" set ");
		for(i=0;i<field.length-1;i++) {
			ret = ret+field[i].name+"="+formatValue(i)+", ";
		}
		ret = ret+field[i].name+"="+formatValue(i);
		if(keys.length>0) {
			ret = ret + " where ";
			for(i=0;i<keys.length-1;i++)  {
				o = _hIndexByFieldName.get(keys[i]);
				if(o==null) { throw new PGexception("updateSQL: ban field name "+keys[i]); }
				j = ((Integer)(o)).intValue();
				ret = ret+field[j].name+"="+formatValue(j)+" and ";
			}
			o = _hIndexByFieldName.get(keys[i]);
			if(o==null) { throw new PGexception("updateSQL: ban field name "+keys[i]); }
			j = ((Integer)(o)).intValue();
			ret = ret+field[j].name+"="+formatValue(j);
		}
		return ret;
	}
	/**
	 * Returns an SQL string that will update a database
	 * table with the current value of this record
	 * for all records having the same values one key field
	 * as this record.
	 * @param sTable - string naming the table to update
	 * 	key - the field name of  "primary key"
	 * @return SQL update string
	 */
	public String updateSQL(String sTable, String key) {
		String sKeys[] = new String [1];
		sKeys[0] = key;
		return updateSQL(sTable, sKeys);
	}
	/**
	 * Returns an SQL string that will insert a record into  a database
	 * table with the current value of this record
	 * @param sTable - string naming the table in which to inserte
	 * @return SQL update string
	 */
	public String insertSQL(String sTable) {
		int i, j;
		String ret = new String("insert into  "+sTable+" ( ");
		for(i=0;i<field.length-1;i++) {
			ret = ret+field[i].name+", ";
		}
		ret = ret+field[i].name+" ) values ( ";

		for(i=0;i<field.length-1;i++) {
			ret = ret+formatValue(i)+", ";
		}
		ret = ret+formatValue(i)+" )";
		return ret;
	}
		
}
	
