/*
    PGresult.java - A Java class to hold Postgres95 results
    Copyright (C) 1996  John F. Kelly

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/
package postgres;

import java.util.*;
/**
 * Class PGresult
 * A standard data structure created by PGconnections
 * to represent results of database operations
 */
public
class PGresult {
	static String _version = "0.2";
	public static String version() { return _version; }
	
	String sCommandName=new String();
	String sPortalName=new String();
	String sErrorMessage=new String();
	Hashtable hFieldByIndex = new Hashtable();
	Hashtable hIndexByFieldName = new Hashtable();
	Hashtable hTupleByIndex = new Hashtable();
	int iFieldCount = 0;
	int iTupleCount = 0;


	public PGresult() {
	}

	/**
	 * Initialize an error result
	 */
	public PGresult(String error) {
		sErrorMessage = error;
	}

	/**
	 * sets the name of the command that generated this result
	 */
	public void setCommandName(String name) {
		sCommandName = name;
	}
	/**
	 * Returns the name of the command that generated this result
	 */
	public String getCommandName() {
		return sCommandName;
	}

	/**
	 * Sets the name of the portal on which this result came
 	 */
	public void setPortalName(String pname) {
		sPortalName = pname;
	}
	/**
	 * Returns the name of the portal on which this result came
 	 */
	public String getPortalName() {
		return sPortalName;
	}
	/** 
	 * Gets any error message
	 */
	public String getErrorMessage() {
		return sErrorMessage;
	}
	/**
	 * Sets an error message
	 */
	public void setErrorMessage(String error) {
		sErrorMessage = error;
	}

	/**
	 * Adds a field to the result set at the given index
	 * @param i - index of field, field- PGfield carrying field info
	 */
	public void addField(int i, PGfield field) {
		Integer ii = new Integer(i);
		hIndexByFieldName.put(field.name, ii);
		hFieldByIndex.put(ii, field);
		iFieldCount++;
	 }

	/** 
	 * Gets the index of a given field name
	 * @param name - String name of field
	 * @exception PGexception if field name not known
	 */
	public int getFieldNameIndex(String name) {
		Object o;
		o = hIndexByFieldName.get(name);
		if(o==null) { throw new PGexception("getFieldNameIndex: unknown field "+name); }
		return ((Integer)(o)).intValue();
	 }

	/** 
	 * Returns the PGfield assocaited with the given index
	 */
	public String getFieldName(int i) {
		return (getField(i)).name;
	 }

	/** 
	 * Returns the PGfield assocaited with the given index
	 */
	public PGfield getField(int i) {
		PGfield pgf = (PGfield)(hFieldByIndex.get(new Integer(i)));
		if(pgf==null) {
			throw new PGexception("getFieldName: bad index "+i);
		} else {
			return pgf;
		}
	 }

	/** 
	 * Returns an array of PGfields assocaited with the result
	 */
	public PGfield[] getFields() {
		PGfield ret[] = new PGfield[iFieldCount];
		int i;
		for(i=0;i<iFieldCount;i++) {
			ret[i] = getField(i);
		}	
		return ret;
	}

	/**
	 * Returns an array of Field names associated with the result
	 */
	public String[] getFieldNames() {
		String ret[] = new String[iFieldCount];
		int i;
		for(i=0;i<iFieldCount;i++) {
			ret[i] = (getField(i)).name;
		}	
		return ret;
	}

	/**
	 * Used by PGconnection to add a tuple to the result set
	 */
	public void addTuple(int i, String tuple[]) {
		hTupleByIndex.put(new Integer(i),tuple);
		iTupleCount++;
	 }

	/** 
	 * Gets the i-th tuple in the result set
	 */
	public String[] getTuple(int i) {
		if(i>=iTupleCount || i<0) { throw new PGexception("getField: index "+ i+ " out of bounds"); }
		return (String[])hTupleByIndex.get(new Integer(i));
	 }

	/**
	 * Returns the number of tuples in the result set
	 */
	public int getTupleCount() {
		return iTupleCount;
	}

	/**
	 * Returns the named field of the given tuple
	 */
	public String getField(int i, String name) {
		return(String) (getTuple(i))[getFieldNameIndex(name)];
	}

	/**
	 * Prints out the tuple and field info
	 * Used mainly for debugging
	 */
	public void printTuples() {
		int i,j;
		for(i=0;i<iFieldCount;i++) {
			PGfield f = getField(i);
			System.out.print(f.name+" ("+f.type+") \t");
		}
		System.out.println();
		for(i=0;i<iTupleCount;i++) {
			String tup[] = getTuple(i);
			for(j=0;j<iFieldCount;j++) {
				System.out.print(tup[j]+"\t");
			}
			System.out.println();
		}
		System.out.println("COMMAND: "+sCommandName);
		System.out.println("Portal: "+sPortalName);
		System.out.println("Errors: "+sErrorMessage);
				
	}
	
	/**
	 * Returns a Enumerable Vector of <a href="postgres.PGrecord.html#top">
	 * PGrecords </a>
	 */
	public Vector getPGvector() {
		PGfield field[] = getFields();
		Vector ret = new Vector(iTupleCount);
		int i;
	
		for(i=0;i<iTupleCount;i++) {
			ret.addElement(new PGrecord(field,hIndexByFieldName,getTuple(i)));
		}
		return ret;
	}
	
}

